library init_screen;

import 'package:flutter/material.dart';
import 'package:material_design_icons_flutter/material_design_icons_flutter.dart';

/// An easy to use, multipurpose init screen to introduce your app
class InitScreen extends StatefulWidget{

    final GlobalKey key;
    final Icon stepIcon;
    final List<Widget> children;
    /// prevent user from going a step back
    final bool noGoBack;
    /// should not be changed
    final int sensitivity;
    final bool showNavHelpers;

    /// those two must not be true at the same time
    final bool showNavButtons;
    final bool showNavBar;

    final Text navHelperEndText;
    final Icon navHelperIconLeft;
    final Icon navHelperIconRight;
    final Function customNavButtonBuilder;
    final Color backgroundColor;
    /// color of the static nav bar
    final Color barColor;
    final Color navBtnFocusedColor;
    final Color navBtnUnfocusedColor;
    final GlobalKey<FormState> validatorKey;
    final FloatingActionButtonLocation navBtnLocation;

    InitScreen({
        @required this.children,
        this.key,
        this.stepIcon = const Icon(MdiIcons.circle),
        this.noGoBack = false,
        this.sensitivity = 1000,
        this.showNavButtons = true,
        this.showNavHelpers = false,
        this.showNavBar = false,
        this.navHelperEndText = const Text("DONE", style: TextStyle(color: Colors.white)),
        this.navHelperIconLeft = const Icon(Icons.chevron_left),
        this.navHelperIconRight = const Icon(Icons.chevron_right),
        this.customNavButtonBuilder,
        this.backgroundColor = Colors.transparent,
        this.barColor = Colors.black,
        this.navBtnFocusedColor = Colors.white,
        this.navBtnUnfocusedColor = Colors.grey,
        this.navBtnLocation = FloatingActionButtonLocation.centerDocked,
        this.validatorKey
    }) : assert(navHelperEndText.data.length < 8), assert(!(showNavBar && showNavButtons)), super(key: key);

    @override
    State<StatefulWidget> createState() => InitScreenState();

}

class InitScreenState extends State<InitScreen> {

    int _currentIndex = 0;

    // build single step button
    _buildStepButton(int index) {
        return widget.customNavButtonBuilder != null ? widget.customNavButtonBuilder(index) : IconButton(
            color: _currentIndex == index ? widget.navBtnFocusedColor : widget.navBtnUnfocusedColor,
            onPressed: () => navBtnClick(index),
            icon: widget.stepIcon,
        );
    }

    // build step button list
    _buildStepButtonList() {
        return List<Widget>.generate(
            widget.children.length,
            (index) => _buildStepButton(index)
        );
    }

    // build NavButton container
    _buildNavBtnRow() {
        return Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
                if (!widget.noGoBack && widget.showNavHelpers)
                    SizedBox(
                        width: 40,
                        child: IconButton(
                            icon: widget.navHelperIconLeft,
                            onPressed: () => navBtnClick(_currentIndex - 1),
                        )
                    )
                else
                    SizedBox(
                        width: 30,
                    ),
                Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: _buildStepButtonList(),
                ),
                if (widget.showNavHelpers)
                    if (_currentIndex == widget.children.length - 1)
                        SizedBox(
                            width: 40,
                            child: widget.navHelperEndText,
                        )
                    else
                        SizedBox(
                            width: 40,
                            child: IconButton(
                                icon: widget.navHelperIconRight,
                                onPressed: () => navBtnClick(_currentIndex + 1),
                            ),
                        )
                else
                    SizedBox(
                        width: 40,
                    )
            ],
        );
    }

    @override
    build(BuildContext context) {
        return SafeArea(
            child: Scaffold(
                resizeToAvoidBottomPadding: false,
                body: GestureDetector(
                    behavior: HitTestBehavior.translucent,
                    onHorizontalDragEnd: _userSwiped,
                    child: SizedBox(
                        height: MediaQuery.of(context).size.height,
                        width: MediaQuery.of(context).size.width,
                        child: Container(
                            color: widget.backgroundColor,
                            child: Stack(
                                children: [
                                    Center(
                                        child: widget.children[_currentIndex]
                                    ),
                                    if (!widget.showNavButtons && widget.showNavBar)
                                        Positioned(
                                            bottom: 0,
                                            child: Container(
                                                color: widget.barColor,
                                                height: 56,
                                                width: MediaQuery.of(context).size.width,
                                                child: _buildNavBtnRow()
                                            ),
                                        )
                                ],
                            ),
                        ),
                    ),
                ),
                floatingActionButtonLocation: widget.navBtnLocation,
                floatingActionButton: widget.showNavButtons && !widget.showNavBar ?
                    _buildNavBtnRow() :
                    null,
            ),
        );
    }

    // public callback method
    navBtnClick(int index) {
        if (widget.validatorKey?.currentState?.validate() ?? true) {
            if (widget.noGoBack && index < _currentIndex)
                return;
            setState(() {
                _currentIndex = index;
            });
        }
    }

    _userSwiped(DragEndDetails details) {
        // yeah...
        if (widget.validatorKey?.currentState?.validate() ?? true) {
            if (details.primaryVelocity.abs() < widget.sensitivity)
                return;
            var newIndex = _currentIndex + (details.primaryVelocity > 0 ? -1 : 1);
            if (widget.noGoBack && details.primaryVelocity > 0)
                return;
            if (newIndex > widget.children.length - 1)
                return;
            if (newIndex < 0)
                return;
            setState(() {
                _currentIndex = newIndex;
            });
        }
    }

}
